c **********************************************************************
c *   Specification of the common and HPF statements                   *
c *                                                                    *
c *   Variables:                                                       *
c *     u, v, w       : Velocity components                     (m/s)  *
c *     t             : Potential temperature                     (K)  *
c *                   : or Liquid water potential temperature          *
c *     q             : Specific humidity                     (kg/kg)  *
c *                   : or Total water content                         *
c *     ql            : Liquid water content                  (kg/kg)  *
c *     pd            : Dynamic pressure divided by density (m^2/s^2)  *
c *     tk            : Turbulent kinetic energy            (m^2/s^2)  *
c *     fm, fh        : Eddy viscosity for momentum & heat    (m^2/s)  *
c *     pi            : Perturbed Exner function             (J/kg K)  *
c *                   : for hydrostatic pressure                       *
c *     pi0           : Reference Exner function             (J/kg K)  *
c *     p             : Pressure                                (hPa)  *
c *      (pi, pi0, p) : These are used in physical processes.          *
c *     vt, vq        : Functions in the condensation process          *
c *     cld           : Cloud fraction                      (0.0-1.0)  *
c *     rhr           : Heating rate due to radiation           (K/s)  *
c *     rfx           : Net downward longwave radiative flux  (W/m^2)  *
c *     sa            : Scalar quantity, e.g., Dust, NOx               *
c *                                                                    *
c *     el            : Turbulent length scale                    (m)  *
c *                   : It is equal to an effective grid spacing.      *
c *     bv            : Buoyant force                         (m/s^2)  *
c *     wo            : Work array                                     *
c *     uu, ..., wu   : Fluxes (Advection + Diffusion)                 *
c *     fub, ..., fab : Right-hand side terms at the previous step     *
c *     wsb           : Time series of the surface velocity     (m/s)  *
c *                                                                    *
c *     flm           : Momentum flux at the surface            (m/s)  *
c *                   : when multiplied by u or v                      *
c *     flt, flq      : Sensible & latent heat fluxes at the surface   *
c *                   : positive upward                  (K m/s, m/s)  *
c *     grm, grh      : Functions for obtaining gradients     (s/m^2)  *
c *     ts            : Soil surface temperature                  (K)  *
c *     fla           : Flux of a scalar quantity at the surface       *
c *                                                                    *
c *     gam, fwu, fvw : Functions in the SGS model of Sullivan et al.  *
c *                   : (1994)                                         *
c *     ug, vg        : Geostrophic velocity components         (m/s)  *
c *     ui, ..., qi   : Initial vertical distributions                 *
c *                                                                    *
c *     tref          : Reference temperature                     (K)  *
c *     gamt, gamq    : Vertical gradients of t and q at the top       *
c *                   : boundary                       (K/m, kg/kg m)  *
c *     fs, fc        : Coriolis parameters                  (s^[-1])  *
c *                                                                    *
c *   # Temperatures (t, ts, ti) are given by differences from tref.   *
c **********************************************************************
c
      include  "Implicit.h"
      include  "Parameter.h"
c
      common  /v3d/ u  (mx,mz,my), v  (mx,mz,my), w  (mx,mz,my),
     :              t  (mx,mz,my), q  (mx,mz,my), ql (mx,mz,my),
     :              pd (mx,mz,my), tk (mx,mz,my),
     :              fm (mx,mz,my), fh (mx,mz,my),
     :              pi (mx,mz,my), pi0(mx,mz,my), p  (mx,mz,my),
     :              vt (mx,mz,my), vq (mx,mz,my), cld(mx,mz,my),
     :              rhr(mx,mz,my), rfx(mx,mz,my),
     :              sa (mx,mz,my),
     :              el (mx,mz,my), bv (mx,mz,my), wo (mx,mz,my),
     :              uu (mx,mz,my), vv (mx,mz,my), ww (mx,mz,my),
     :              uv (mx,mz,my), vw (mx,mz,my), wu (mx,mz,my),
     :              fub(mx,mz,my), fvb(mx,mz,my), fwb(mx,mz,my),
     :              ftb(mx,mz,my), fqb(mx,mz,my), wsb(mx,mz,my),
     :              fab(mx,mz,my)
      common  /v2d/ flm(mx,   my), flt(mx,   my), flq(mx,   my),
     :              grm(mx,   my), grh(mx,   my), ts (mx,   my),
     :              fla(mx,   my)
      common  /v1d/ gam(   mz   ), fwu(   mz   ), fvw(   mz   ),
     :              ui (   mz   ), vi (   mz   ),
     :              ti (   mz   ), qi (   mz   ),
     :              ug ( 0:mz   ), vg ( 0:mz   )
      common  /cns/   dt,   dx,   dy,   dz, tref, gamt, gamq,
     :                fs,   fc,
     :                 g,   rd,   cp,   p0,   t0,   ev,
     :              csgs,  ctk,  tpr,  anu,  tnu,
     :               gmm,  gmh,  aph,  spr,   vk,
     :               gno,  gpw
c
!hpf$ distribute (*,*,block) ::
!hpf$:          u,   v,   w,   t,   q,  ql,  pd,  tk,  fm,  fh,
!hpf$:         pi, pi0,   p,  vt,  vq, cld, rhr, rfx,  sa,
!hpf$:         el,  bv,  wo,  uu,  vv,  ww,  uv,  vw,  wu,
!hpf$:        fub, fvb, fwb, ftb, fqb, wsb, fab
c
!hpf$ distribute   (*,block) ::
!hpf$:        flm, flt, flq, grm, grh,  ts, fla
