c **********************************************************************
c *   Sponge layer for avoiding reflections of gravity waves           *
c *                                                                    *
c *   Input variables:                                                 *
c *     nspl          : Number of the sponge layers near the bottom    *
c *     nspu          : Number of the sponge layers near the top       *
c *                   : = 0,       No sponge layer is placed.          *
c *     csp           : Rayleigh damping coefficient at the boundary   *
c *                                                                    *
c *   Description:                                                     *
c *     This scheme is as follows:                                     *
c *       u[*]   = u[n] + a,                  in sub.predict           *
c *       u[n+1] = u[*] - c ( u[n+1] - ui ),                           *
c *       Hence  u[n+1] = ( u[*] + c ui ) / ( 1 + c ),                 *
c *     According to Klemp and Lilly (1978), 2 < csp/k ui < 5, where   *
c *     k is the horizontal wavenumber.                                *
c *     Normally csp seems to have a value of 0.001-0.03.              *
c **********************************************************************
c
      subroutine  sponge_layer ( nspl, nspu, csp, icon )
c
      include  "Declare.h"
c
      hpai = 1.570796
c
      zupp = dz*  nz
c
      kzl  =     nspl
      kzu  = nz -nspu +1
      zkzl = dz*  kzl
      zkzu = dz*( kzu-1.0 )
c
      do j = 2,ny-1
c
      do k = 1,kzl
        zok = dz*( k-0.5 )
        zwk = dz*( k-1.0 )
        cs1 = dt*csp*sin( -hpai*( zok-zkzl )/       zkzl   )**2
        cs2 = dt*csp*sin( -hpai*( zwk-zkzl )/       zkzl   )**2
      do i = 2,nx-1
        u (i,k,j) = ( u (i,k,j)+cs1*ui( k ) ) / ( 1.0+cs1 )
        v (i,k,j) = ( v (i,k,j)+cs1*vi( k ) ) / ( 1.0+cs1 )
c?      u (i,k,j) = ( u (i,k,j)+cs1*ug( k ) ) / ( 1.0+cs1 )
c?      v (i,k,j) = ( v (i,k,j)+cs1*vg( k ) ) / ( 1.0+cs1 )
        w (i,k,j) =   w (i,k,j)               / ( 1.0+cs2 )
        t (i,k,j) = ( t (i,k,j)+cs1*ti( k ) ) / ( 1.0+cs1 )
        q (i,k,j) = ( q (i,k,j)+cs1*qi( k ) ) / ( 1.0+cs1 )
      end do
      end do
c
      do k = kzu,nz
        zok = dz*( k-0.5 )
        zwk = dz*( k-1.0 )
        cs1 = dt*csp*sin(  hpai*( zok-zkzu )/( zupp-zkzu ) )**2
        cs2 = dt*csp*sin(  hpai*( zwk-zkzu )/( zupp-zkzu ) )**2
      do i = 2,nx-1
        u (i,k,j) = ( u (i,k,j)+cs1*ui( k ) ) / ( 1.0+cs1 )
        v (i,k,j) = ( v (i,k,j)+cs1*vi( k ) ) / ( 1.0+cs1 )
c?      u (i,k,j) = ( u (i,k,j)+cs1*ug( k ) ) / ( 1.0+cs1 )
c?      v (i,k,j) = ( v (i,k,j)+cs1*vg( k ) ) / ( 1.0+cs1 )
        w (i,k,j) =   w (i,k,j)               / ( 1.0+cs2 )
        t (i,k,j) = ( t (i,k,j)+cs1*ti( k ) ) / ( 1.0+cs1 )
        q (i,k,j) = ( q (i,k,j)+cs1*qi( k ) ) / ( 1.0+cs1 )
      end do
      end do
c
      end do
c
      call bc_lat_moment
      call bc_upp_moment
c
      call bc_lat_scalar ( t )
      call bc_upp_scalar ( t, gamt*dz )
c
      if ( icon .lt. 0 ) return
      call bc_lat_scalar ( q )
      call bc_upp_scalar ( q, gamq*dz )
c
      return
      end
