c **********************************************************************
c *   Surface fluxes (computed from the prescribed heat flux)          *
c *                                                                    *
c *   Input variables:                                                 *
c *     isfc          : = 1,       for horizontal-mean velocity        *
c *                   : = 2,       for instantaneous one in each grid  *
c *                   : > 2,       for temporal-mean one in each grid  *
c *     istep         : Time-step number                               *
c *                                                                    *
c *     new           : = 0,       in the first call                   *
c *     z0m           : Roughness length for momentum             (m)  *
c *                   : If z0m = 0, the surface is the sea surface.    *
c *     z0h           : Dummy (not used)                               *
c *     tfmx, qfmx    : Maximum fluxes of heat and vapor (K m/s, m/s)  *
c *                   : These are positive upward, and vanish at       *
c *                   : sunrise and sunset.                            *
c *     phour         : Time at which the maximum fluxes occur (hour)  *
c *     dhour         : Length of the daytime                  (hour)  *
c *                   : Sunrise is at phour-(dhour/2) LST.             *
c *                   : If dhour=<0, fluxes are fixed at tfmx & qfmx.  *
c *     chour         : Current time starting from 00 LST      (hour)  *
c *                                                                    *
c *   Output variables (horizontal average):                           *
c *     ust           : Friction velocity                       (m/s)  *
c *     duz           : Vertical shear at a height of dz     (s^[-1])  *
c *                   : These are used only in sub.sgs_model.          *
c *                                                                    *
c *   # Note that zeta has a muximum according to log((z+z0m)/z0m)     *
c *     and alpha, which is equal to a double root, since its root     *
c *     changes to a complex number with increasing stability.         *
c **********************************************************************
c
      subroutine  surface_prescript ( isfc, istep,
     :                new, z0m, z0h, tfmx, qfmx, phour, dhour, chour,
     :                ust, duz )
c
      include  "Declare.h"
c
      data  hpai/1.570796/
      data  wmin,zm2h,elmn/ 0.01, 7.40, 0.10/
c
      interface
        pure subroutine psi_phi
     :             ( sb, alm, zz0, zzw, gmm, gmh, aph, spr,
     :               psm, phm, phh, phw,  sbv, tvus )
          include  "Implicit.h"
          intent (in)   sb, alm, zz0, zzw, gmm, gmh, aph, spr, sbv
          intent (out)  psm, phm, phh, phw, tvus
        end subroutine
      end interface
c
      z1   = 0.5 *dz
      rsum = 1.0 /dble((nx-2)*(ny-2))
c
      if ( dhour .gt. 0.0 ) then
        tsus = -tfmx*cos( hpai*( chour-phour )/( 0.5*dhour ) )
        qsus = -qfmx*cos( hpai*( chour-phour )/( 0.5*dhour ) )
      else
        tsus = -tfmx
        qsus = -qfmx
      end if
c
      wuav = 0.0
      wvav = 0.0
      vtav = 0.0
      vqav = 0.0
c
!hpf$ independent, new(i), reduction(wuav, wvav, vtav, vqav)
      do j = 2,ny-1
      do i = 2,nx-1
        wuav = wuav + u (i,1,j)
        wvav = wvav + v (i,1,j)
        vtav = vtav + vt(i,1,j) +0.61*  q (i,1,j) +1.0
        vqav = vqav + vq(i,1,j) +0.61*( t (i,1,j) +tref )
      end do
      end do
c
      wuav = wuav * rsum
      wvav = wvav * rsum
      vtav = vtav * rsum
      vqav = vqav * rsum
      ws1  = max( sqrt( wuav**2+wvav**2 ), wmin )
c
c  ***  ust over the sea surface  ***
      if ( new .eq. 0 ) then
        ust = ws1 * 0.05
        a   = log( z1*g/0.016    )
        b   = log( z1  /0.165e-5 )
c
        do iter = 1,5
          if ( ust .gt. 0.1 ) then
            f  = ust*( a-2.0*log( ust ) ) -vk*ws1
            df = a -2.0 -2.0*log( ust )
          else
            f  = ust*( b+    log( ust ) ) -vk*ws1
            df = b +1.0 +    log( ust )
          end if
c
          ust = ust -f/df
        end do
      end if
c
c  ***  Land surface  ***
      if ( z0m .gt. 0.0 ) then
                            z0mm = z0m
cx                          z0hh = z0m/zm2h
cx      if ( z0h .gt. 0.0 ) z0hh = z0h
c
c  ***  Sea surface  ***
      else
        if ( ust .gt. 0.1 ) then
                            z0mm = 0.016   *ust**2/g
        else
                            z0mm = 0.165e-5/ust
        end if
cx                          z0hh = 5.0e-5
      end if
c
      zz  = z1 +z0mm
      alm = log( zz/z0mm )
cx    alh = log( zz/z0hh ) * spr
      zz0 =      z0mm  /zz
      zzw = ( dz+z0mm )/zz
c
      sbo   = g * zz / ( tref * vk**2 )
      tvusa = vtav*tsus + vqav*qsus
c
      sbv = sbo / ws1**3
      sb  = sbv * tvusa
c
      call psi_phi ( sb, alm, zz0, zzw, gmm, gmh, aph, spr,
     :               psm, phm, phh, phw,  sbv, tvus )
c
      ust = vk *ws1/  psm
      duz = ust*phw/( vk*( dz+z0mm ) )
c
      if ( tvus .gt. 0.0 ) then
        tvusa = tvus
c
        if ( tsus .eq. 0.0 ) then
          qsus = tvus /  vqav
        else
          t2q  = qsus /  tsus
          tsus = tvus /( vtav + vqav*t2q )
          qsus = tsus *  t2q
        end if
      end if
c
      if ( isfc .eq. 1 ) then
c
!hpf$ independent, new(i)
        do j = 2,ny-1
        do i = 2,nx-1
          flm(i,j) = -ust**2/ws1
          flt(i,j) = -tsus
          flq(i,j) = -qsus
          grm(i,j) =  phm / ( vk*zz*ust )
          grh(i,j) =  phh / ( vk*zz*ust )
        end do
        end do
c
      else
        if ( isfc .eq. 2 ) then
          do j = 2,ny-1
          do i = 2,nx-1
            wsb(i,mz,j) = sqrt( u (i,1,j)**2+v (i,1,j)**2 )
          end do
          end do
        else
          call time_average ( istep )
        end if
c
!hpf$ independent, new(i)
        do j = 2,ny-1
        do i = 2,nx-1
          ws1 = max( wsb(i,mz,j), wmin )
c
          sbv = sbo / ws1**3
          sb  = sbv * tvusa
c
!CDIR IEXPAND
          call psi_phi ( sb, alm, zz0, zzw, gmm, gmh, aph, spr,
     :               psm, phm, phh, phw,  sbv, tvus )
c
          us1 = vk *ws1/psm
c
c  ***  tsus and qsus should be reevaluated as above?  ***
          flm(i,j) = -us1**2/ws1
          flt(i,j) = -tsus
          flq(i,j) = -qsus
          grm(i,j) =  phm / ( vk*zz*us1 )
          grh(i,j) =  phh / ( vk*zz*us1 )
        end do
        end do
c
      end if
c
      call bc_lat_scalar2 ( flm )
c
      return
      end
c
      pure subroutine  psi_phi
     :             ( sb, alm, zz0, zzw, gmm, gmh, aph, spr,
     :               psm, phm, phh, phw,  sbv, tvus )
c
      include  "Implicit.h"
      intent (in)   sb, alm, zz0, zzw, gmm, gmh, aph, spr, sbv
      intent (out)  psm, phm, phh, phw, tvus
c
c  ***  aph_k = 6.0 is based on Kondo (1975, BLM, 9, 91-112)  ***
c?    aph_k = aph
      aph_k = 6.0
c
      tvus = -99.0
c
      if     ( sb .eq. 0.0 ) then
        psm = alm
        phm = 1.0
        phh = spr
        phw = 1.0
c
      elseif ( sb .gt. 0.0 ) then
        zet = 1.0e5
c
!CDIR UNROLL=5
        do iter = 1,5
          phm = 1.0 +aph_k*zet    /( 1.0+zet     )
          p0m = 1.0 +aph_k*zet*zz0/( 1.0+zet*zz0 )
c
          psm = alm +aph_k*log( (1.0+zet)/(1.0+zet*zz0) )
c
          f   = sb*psm**3 -zet
          df  = sb*psm**2 *3.0*( phm-p0m )/zet -1.0
          zet = zet -f/df
c
          if ( zet .le. 0.0  ) zet =  1.0e-10
        end do
c
        phm = 1.0 + aph_k*zet    /( 1.0+zet     )
        phh = spr + aph_k*zet    /( 1.0+zet     )
        phw = 1.0 + aph_k*zet*zzw/( 1.0+zet*zzw )
c
      else
        zet = sb * alm**3
c
!CDIR UNROLL=5
        do iter = 1,5
          rhm = ( 1.0-gmm*zet     )**0.25
          r0m = ( 1.0-gmm*zet*zz0 )**0.25
c
          psm = alm -log( ( rhm+1.0 )**2*( rhm**2+1.0 )
     :                 /( ( r0m+1.0 )**2*( r0m**2+1.0 ) ) )
     :         +2.0*( atan( rhm )-atan( r0m ) )
c
          f   = sb*psm**3 -zet
          df  = sb*psm**2 *( 3.0/rhm -3.0/r0m )/zet -1.0
          zet = zet -f/df
c
          if ( zet .ge. 0.0  ) zet = -1.0e-10
        end do
c
        phm = 1.0/    ( 1.0-gmm*zet     )**0.25
        phh = spr/sqrt( 1.0-gmh*zet     )
        phw = 1.0/    ( 1.0-gmm*zet*zzw )**0.25
c
      end if
c
      return
      end
